@@ -14,6 +14,7 @@ from typing import (
     Set,
     Tuple,
     OrderedDict,
+    List,
 )
 
 import docutils.nodes
@@ -759,6 +760,12 @@ class Method(SignatureLike):
 
     parsed: parse.Method
 
+    #: The original class where this method is specified.
+    #: We stack all the methods over the ancestors, so using ``specified_for``
+    #: you can distinguish between inherited methods and genuine methods of
+    #: a class.
+    specified_for: Final["Class"]
+
     # fmt: off
     @require(
         lambda name:
@@ -797,6 +804,7 @@ class Method(SignatureLike):
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
         description: Optional[SignatureDescription],
+        specified_for: "Class",
         contracts: Contracts,
         parsed: parse.Method,
     ) -> None:
@@ -811,6 +819,8 @@ class Method(SignatureLike):
             parsed=parsed,
         )
 
+        self.specified_for = specified_for
+
     @abc.abstractmethod
     def __repr__(self) -> str:
         # Signal that this is a pure abstract class.
@@ -865,6 +875,7 @@ class UnderstoodMethod(Method):
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
         description: Optional[SignatureDescription],
+        specified_for: "Class",
         contracts: Contracts,
         body: Sequence[parse_tree.Node],
         parsed: parse.Method,
@@ -876,6 +887,7 @@ class UnderstoodMethod(Method):
             arguments=arguments,
             returns=returns,
             description=description,
+            specified_for=specified_for,
             contracts=contracts,
             parsed=parsed,
         )
@@ -1219,6 +1231,29 @@ class Class(DBC):
         """Collect IDs (with :py:func:`id`) of the inheritance objects in a set."""
         return self._inheritance_id_set
 
+    def is_subclass_of(self, cls: "ClassUnion") -> bool:
+        """
+        Check recursively whether this class is a sub-class of ``cls``.
+
+        Every class is a sub-class of itself.
+        """
+        # NOTE (mristin, 2022-05-13):
+        # This function is not used by the aas-core-codegen, but by downstream clients
+        # such as aas-core3.0rc02-testgen.
+
+        if id(cls) == id(self):
+            return True
+
+        queue = [self]  # type: List[Class]
+        while len(queue) > 0:
+            top = queue.pop()
+            if id(cls) in top.inheritance_id_set:
+                return True
+
+            queue.extend(top.inheritances)
+
+        return False
+
     # endregion
 
     #: If set, this class is implementation-specific and we need to provide a snippet
@@ -1768,6 +1803,7 @@ class Interface:
                 parsed=method.parsed,
             )
             for method in base.methods
+            if method.specified_for is base
         ]
 
         self.description = base.description
